#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <iostream>
#include "wcam.h"
#include "types.h"
#include "vsocket.h"
#include <stdio.h>

U16 	width;
U16 	height;
U16 	tmark;
U8	ocvbuf[_szvbuf];

CvCapture* capture;

pthread_t	hCamLoop;

//тестовый интерактивеый модуль
int test(void)
{
	int	num = 0;
	//
	capture = cvCreateCameraCapture(0);
	if(!(capture))
		return 1;
	while(1)
	{
		cv::Mat src = cvQueryFrame(capture);

    		cv::imshow("src", src);

    		// Create binary image from source image
    		cv::Mat bw;
    		cv::cvtColor(src, bw, CV_BGR2GRAY);
    		cv::threshold(bw, bw, 40, 255, CV_THRESH_BINARY);
		if(num == 0)
			cv::imshow("out", bw);

    		// Perform the distance transform algorithm
    		cv::Mat dist;
    		cv::distanceTransform(bw, dist, CV_DIST_L2, 3);

    		// Normalize the distance image for range = {0.0, 1.0}
    		// so we can visualize and threshold it
    		cv::normalize(dist, dist, 0, 1., cv::NORM_MINMAX);
		if(num == 1)
			cv::imshow("out", dist);

    		// Threshold to obtain the peaks 
    		// This will be the markers for the foreground objects
    		cv::threshold(dist, dist, .5, 1., CV_THRESH_BINARY);
		if(num == 2)
			cv::imshow("out", dist);

    		// Create the CV_8U version of the distance image
    		// It is needed for cv::findContours()
    		cv::Mat dist_8u;
    		dist.convertTo(dist_8u, CV_8U);

    		// Find total markers
   	 	std::vector<std::vector<cv::Point> > contours;
   		cv::findContours(dist_8u, contours, CV_RETR_EXTERNAL, CV_CHAIN_APPROX_SIMPLE);
    		int ncomp = (int) contours.size();

    		// Create the marker image for the watershed algorithm
    		cv::Mat markers = cv::Mat::zeros(dist.size(), CV_32SC1);

    		// Draw the foreground markers
    		for (int i = 0; i < ncomp; i++)
			cv::drawContours(markers, contours, i, cv::Scalar::all(i+1), -1);

    		// Draw the background marker
    		cv::circle(markers, cv::Point(5,5), 3, CV_RGB(255,255,255), -1);
		if(num == 3)
			cv::imshow("out", markers*10000);

    		// Perform the watershed algorithm
    		cv::watershed(src, markers);

    		// Generate random colors
    		std::vector<cv::Vec3b> colors;
    		for (int i = 0; i < ncomp; i++)
    		{
	    		int b = cv::theRNG().uniform(0, 255);
	    		int g = cv::theRNG().uniform(0, 255);
	    		int r = cv::theRNG().uniform(0, 255);

	    		colors.push_back(cv::Vec3b((uchar)b, (uchar)g, (uchar)r));
    		}

    		// Create the result image
    		cv::Mat dst = cv::Mat::zeros(markers.size(), CV_8UC3);

    		// Fill labeled objects with random colors
    		for (int i = 0; i < markers.rows; i++)
    		{
			for (int j = 0; j < markers.cols; j++)
			{
	    		int index = markers.at<int>(i,j);
        		dst.at<cv::Vec3b>(i,j) = index > 0 && index <= ncomp ? colors[index - 1] : cv::Vec3b(0, 0, 0);
			}
    		}
		if(num == 4)
		cv::imshow("out", dst);
		//gray
		cv::Mat gray;
    		cv::cvtColor(src, gray, CV_BGR2GRAY);
		if(num == 5)
			cv::imshow("out", gray);
		//adaptive
		cv::Mat adapt = cv::Mat::zeros(markers.size(), CV_8UC3);
		cv::adaptiveThreshold(gray, adapt, 255, CV_ADAPTIVE_THRESH_MEAN_C, CV_THRESH_BINARY, 11, 1);
		if(num == 6)
			cv::imshow("out", adapt);
		// Find total markers
    		std::vector<std::vector<cv::Point> > acontours;
    		cv::findContours(adapt, acontours, CV_RETR_EXTERNAL, CV_CHAIN_APPROX_SIMPLE);
    		int ancomp = (int) acontours.size();

    		// Create the marker image for the watershed algorithm
    		cv::Mat amarkers = cv::Mat::zeros(dist.size(), CV_32SC1);

    		// Draw the foreground markers
    		for (int i = 0; i < ancomp; i++)
			cv::drawContours(amarkers, acontours, i, cv::Scalar::all(i+1), -1);
		// Perform the watershed algorithm
    		cv::watershed(src, amarkers);
		if(num == 7)
			cv::imshow("out", amarkers*10000);

    		//cv::waitKey(0);
    		char c = cvWaitKey(33);
    		if (c == 27) { // нажата ESC
        		break; }
		else if(c == 0x31)
			num = 0;
		else if(c == 0x32)
			num = 1;
		else if(c == 0x33)
			num = 2;
		else if(c == 0x34)
			num = 3;
		else if(c == 0x35)
			num = 4;
		else if(c == 0x36)
			num = 5;
		else if(c == 0x37)
			num = 6;
		else if(c == 0x38)
			num = 7;

	}  
	return 0;
}

//информация о параметрах камеры
int info(void)
{
	capture = cvCreateCameraCapture(0);
	if(!(capture))
		return 1;
	width = (int)(cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_WIDTH));
        height = (int)(cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_HEIGHT));
	//
	cvReleaseCapture(&capture);
	return 0;
}

//запуск процесса камеры
int CamStart(void)
{
	pthread_attr_t 	tattr;
	//параметры для отделяемых потоков
	pthread_attr_init(&tattr);
	pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED);
	//
	capture = cvCreateCameraCapture(0);
	if(!(capture))
		return 1;
	width = (int)(cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_WIDTH));
        height = (int)(cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_HEIGHT));
	//
	//pthread_create(&hCamLoop, NULL,  CamLoop, NULL);	//не отделяемый поток
	pthread_create(&hCamLoop, &tattr,  CamLoop, NULL);	//отделяемый поток
	//
	//
	return 0;
}

//рабочий процесс камеры
void* CamLoop(void * pr)
{
	U32	cntx, cnty, cntbyte;
	cv::Mat src;
	cv::Mat gray;
	tmark = 0;
	//
	printf("start camm loop...\n");
	//
	while(1)
	{
		src = cvQueryFrame(capture);
		cv::cvtColor(src, gray, CV_BGR2GRAY);
		cnty 	= 0;
		cntbyte	= 0;
		pthread_mutex_lock(&ServerMutex);
		while(cnty < height)
		{
			cntx = 0;
			while(cntx < width)
			{
				ocvbuf[cntbyte] = gray.at<char>(cnty,cntx);
				cntx++;
				cntbyte++;
			}
			cnty++;
		}
		tmark++;
		pthread_mutex_unlock(&ServerMutex);
		usleep(_wait_cam);
	}
	//
	pthread_exit(0);
}

